/* fg.c -- fg builtin */

/* Copyright (C) 2003 Mike Thielman 
 *
 * This file is part of pr0nsh, the Porn Shell.
 *
 * Pr0nsh is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2, or (at your option) any later
 * version.
 *
 * Pr0nsh is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Pr0nsh; see the file COPYING.  If not, write to the Free Software
 * Foundation, 59 Temple Place, Suite 330, Boston, MA 02111 USA. */

#include <stdio.h>
#include <sys/types.h>
#include <sys/wait.h>
#include "util/llist.h"
#include "util/debug.h"
#include "shell/process.h"
#include "shell/redir.h"
#include "shell/shell.h"

/*****************************************************************************
 * Shell built-in fg
 *
 * Arguments: fg [job id]
 *   job id - The job you wish to put in the fg
 *
 * This will put the most recently executed job
 * in the foreground if no argument is specified.
 *****************************************************************************/
int _builtin_fg(LinkedList *args)
{
  int jobid,stat;
  pid_t pid;
  Job *job;

  if(getNumJobs() == 0) {
    outputerr("There are no jobs to put in the foreground\n");
    return RET_ERROR;
  }
  if(args) {
    jobid = atoi(args->s);
    job = getJobStructFromJobID(jobid);
    if(job == NULL) {
      outputerr("Invalid job id %d specified\n", jobid);
      return RET_ERROR;
    }
    stat = getJobStatus(job);
    if(stat != JOB_STOPPED && stat != JOB_SUSPENDED &&
       stat != JOB_RUNNING) {
      outputerr("Job %d can't be foregrounded\n", jobid);
      return RET_ERROR;
    }
  }
  else {
    // A job was not specified, so we want to find the most recent
    // job that is still running.
    job = getNextJob(NULL);
    while(job)
    {
      stat = getJobStatus(job);
      if(stat == JOB_STOPPED || stat == JOB_SUSPENDED ||
         stat == JOB_RUNNING) jobid = getJobNum(job);
      job=getNextJob(job);
    }
    if(jobid == 0) {
      outputerr("There are no jobs to put in foreground\n");
      return RET_ERROR;
    }
  }
  // At this point we want to put "job" in the foreground.
  // If it is not running, send it SIGCONT
  // Then, remove from job list and call _exec_wait

  doutput("Put job %d in the foreground\n", jobid);
  job = getJobStructFromJobID(jobid);
  pid = getJobPID(job);
  outputout("%s\n", getJobCmdline(job));
  setCurrentAST(job->ast);
  RemoveJob(job);  // Remove from list of jobs
  if(stat == JOB_STOPPED || stat == JOB_SUSPENDED) {
    kill(pid,SIGCONT);
  }
  return _exec_wait(pid);
}
