/* lex.l -- Lexer to go along with parse.y */

/* Copyright (C) 2003 Mike Thielman 
 *
 * This file is part of pr0nsh, the Porn Shell.
 *
 * Pr0nsh is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2, or (at your option) any later
 * version.
 *
 * Pr0nsh is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Pr0nsh; see the file COPYING.  If not, write to the Free Software
 * Foundation, 59 Temple Place, Suite 330, Boston, MA 02111 USA. */

%{
#include <readline/readline.h>
#include <readline/history.h>
#include <string.h>

#include "pr0nsh.h"
#include "util/llist.h"
#include "util/symbol.h"
#include "util/debug.h"
#include "shell/shell.h"
#include "shell/redir.h"
#include "shell/pr0nsh.h"
#include "parse/ast.h"
#include "parse/parse.h"
#include "shell/dir.h"

// FIXME: Not using max_size
// FIXME: buffer overrun with tmp
#define YY_INPUT(buf,result,max_size) \
  { \
    int addToHistory = 1; \
    char *prompt; \
    FILE *fp; \
    char tmp[1000]; \
    if (getExecNext()) { \
      setCurCMD(getExecNext()); \
      setExecNext(NULL); \
    } else if ((fp = getNextFP()) != NULL) { \
      addToHistory = 0; \
      fgets(tmp,sizeof(tmp),fp); \
      if(tmp[strlen(tmp)-1] == '\n') tmp[strlen(tmp)-1] = 0; \
      if(!feof(fp)) setCurCMD(tmp); \
      else { \
        closeCurFP(); \
        setCurCMD(""); \
      } \
    } else { \
      if (!isInteractive()) { setCurCMD(readline(NULL)); } \
      else { \
        prompt = getPrompt(); \
        setCurCMD(readline(prompt)); \
        free(prompt); \
      } \
    } \
    if( getCurCMD() != NULL ) { strcpy(buf,getCurCMD()); \
                                if(strcmp(getCurCMD(),"")) \
                                  if(addToHistory) add_history(getCurCMD()); \
                                addToHistory = 1; \
                                strcat(buf, "\n"); \
                                result = strlen(buf); } \
    else { result = YY_NULL; } \
  }
%}
%%
'[^']+'                   { yylval.str = (char *)mystrdup(yytext+1);
                            yylval.str[strlen(yylval.str)-1] = 0;
                            return STRING; }
\"[^\"]+\"                { yylval.str = (char *)mystrdup(yytext+1);
                            yylval.str[strlen(yylval.str)-1] = 0;
                            return STRING; }
[ \t]+                    { ; }
#.*                       { removeLastHistoryEntry(); }
\$[a-zA-Z0-9_$?]+         { yylval.str = mystrdup(yytext+1); 
                            return VAR; }
\!                        { return BANG; }
\|                        { return PIPE; }
;                         { return CR; }
\n                        { setLineNumber(getLineNumber()+1); 
                            return CR; }
\&                        { return AMP; }
\`                        { return BTICK; }
\>                        { yylval.redir = newREDIR();
                            yylval.redir->stream = 1;
                            yylval.redir->type = REDIR_OUTPUT;
                            yylval.redir->next = NULL;
                            return REDIR_TOK; }
\>\>                      { yylval.redir = newREDIR();
                            yylval.redir->stream = 1;
                            yylval.redir->type = REDIR_OUTPUT_APPEND;
                            yylval.redir->next = NULL;
                            return REDIR_TOK; }
\<                        { yylval.redir = newREDIR();
                            yylval.redir->stream = 0;
                            yylval.redir->type = REDIR_INPUT;
                            yylval.redir->next = NULL;
                            return REDIR_TOK; }
[0-9]+\>                  { yylval.redir = newREDIR();
                            yylval.redir->stream = atoi(yytext);
                            yylval.redir->type = REDIR_OUTPUT;
                            yylval.redir->next = NULL;
                            return REDIR_TOK; }
[0-9]+\>\>                { yylval.redir = newREDIR();
                            yylval.redir->stream = atoi(yytext);
                            yylval.redir->type = REDIR_OUTPUT_APPEND;
                            yylval.redir->next = NULL;
                            return REDIR_TOK; }
[0-9]+\<                  { yylval.redir = newREDIR();
                            yylval.redir->stream = atoi(yytext);
                            yylval.redir->type = REDIR_INPUT;
                            yylval.redir->next = NULL;
                            return REDIR_TOK; }
[a-zA-Z0-9:*/\._~\-=?]+   { yylval.str = expandTilde(yytext);
                            return STRING; }
