/* alias.c -- Functions for dealing with aliases. */

/* Copyright (C) 2003 Mike Thielman 
 *
 * This file is part of pr0nsh, the Porn Shell.
 *
 * Pr0nsh is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2, or (at your option) any later
 * version.
 *
 * Pr0nsh is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Pr0nsh; see the file COPYING.  If not, write to the Free Software
 * Foundation, 59 Temple Place, Suite 330, Boston, MA 02111 USA. */

#include <stdio.h>

#include "pr0nsh.h"
#include "shell/alias.h"
#include "util/debug.h"
#include "util/llist.h"
#include "shell/shell.h"

void _makeAlias(char *cmd,char *alias);
Alias *_aliases;

void _makeAlias(char *cmd,char *alias)
{  
  Alias *curalias;
  if(LookupBuiltin(cmd)) {
    outputerr("alias: Can't alias shell built-in command \"%s\"\n", cmd);
  } else {
    if((curalias = LookupAlias(cmd)) != NULL) {
      free(curalias->alias);
      curalias->alias = strdup(alias);
    }
    else {
      curalias = newAlias();
      curalias->cmd = strdup(cmd);
      curalias->alias = strdup(alias);
      if(getCurrentAliases() == NULL) curalias->next = NULL;
      else { curalias->next = getCurrentAliases(); }
      setCurrentAliases(curalias);
    }
  }
}

Alias *LookupAlias(char *cmd)
{
  Alias *cur = getCurrentAliases();

  while(cur != NULL) {
    if(!strcmp(cmd, cur->cmd)) return cur;
    cur = cur->next;
  }
  return NULL;
}

char *ResolveAlias(char *cmd)
{
  char *prevcmd,*newcmd,*tmpargs,*tmpargs2,*p;
  Alias *a;
  LinkedList *prev = NULL;

  if(a = LookupAlias(cmd)) {
    prev = pushll(prev,cmd);
    prevcmd = mystrdup(cmd);
    newcmd = mystrdup(a->alias);
    p = strchr(newcmd, ' ');
    if(p == NULL) p = strchr(newcmd, '\t');
    if(p != NULL) { *p = 0; tmpargs = mystrdup(p+1); }
    else { tmpargs = NULL; }
    while(a && strcmp(newcmd,prevcmd)) {
      if(searchll(prev,newcmd)) {
        outputerr("Alias loop.\n");
        return NULL;
      }
      if(a = LookupAlias(newcmd)) {
        free(prevcmd);
        prev = pushll(prev,newcmd);
        prevcmd = newcmd;
        newcmd = mystrdup(a->alias);
        p = strchr(newcmd, ' ');
        if(p == NULL) p = strchr(newcmd, '\t');
        if(p != NULL) { 
          *p = 0; 
          if(tmpargs != NULL) {
            tmpargs2 = emalloc(mystrlen(tmpargs) + mystrlen(p+1) + 2);
          }
          else { 
            tmpargs2 = emalloc(mystrlen(p+1)); 
          }
          strcpy(tmpargs2, p+1);
          if(tmpargs != NULL) {
            strcat(tmpargs2, " ");
            strcat(tmpargs2, tmpargs);
          }
          free(tmpargs); 
          tmpargs = tmpargs2;
        }
      }
    }
    p = emalloc(mystrlen(newcmd) + mystrlen(tmpargs) + 2);
    strcpy(p, newcmd);
    if(tmpargs != NULL) {
      strcat(p, " ");
      strcat(p, tmpargs);
    }
    return p;
  }
  else { return NULL; }
}

void freeAlias(Alias *alias)
{
  if(alias) {
    free(alias->cmd);
    free(alias->alias);
    freepp(alias->args);
    freeAlias(alias->next);
    free(alias);
  }
}

Alias *copyAlias(Alias *alias)
{
  Alias *copy = NULL;

  copy = newAlias();
  if(alias->cmd)    copy->cmd = strdup(alias->cmd);
  else              copy->cmd = NULL;
  if(alias->alias)  copy->alias = strdup(alias->alias);
  else              copy->alias = NULL;
  if(alias->args)   copy->args = copypp(alias->args);
  else              copy->args = NULL;
  if(alias->next)   copy->next = copyAlias(alias->next);
  else              copy->next = NULL;

  return copy;
}

Alias *newAlias(void)
{
  Alias *alias;

  alias = (Alias *) emalloc(sizeof(Alias));

  alias->cmd = NULL;
  alias->alias = NULL;
  alias->args = NULL;
  alias->next = NULL;

  return alias;
}
