/* exec.c -- Functions for executing commands. */

/* Copyright (C) 2003 Mike Thielman 
 *
 * This file is part of pr0nsh, the Porn Shell.
 *
 * Pr0nsh is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2, or (at your option) any later
 * version.
 *
 * Pr0nsh is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Pr0nsh; see the file COPYING.  If not, write to the Free Software
 * Foundation, 59 Temple Place, Suite 330, Boston, MA 02111 USA. */

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <glob.h>
#include <errno.h>
#include <signal.h>

#include "util/llist.h"
#include "util/symbol.h"
#include "util/debug.h"
#include "util/charpp.h"
#include "parse/ast.h"
#include "shell/dir.h"
#include "shell/exec.h"
#include "shell/alias.h"
#include "shell/redir.h"
#include "shell/shell.h"

extern int errno;
int _alias_ok = 1;

double _execute(ASTCommand *ast,int outp);

double executeCmd(ASTCommand *ast)
{
  Symbol *s;
  char *p,*q,*path,*tmppath;
  LinkedList *_cmdline;
  char *cmdname;
  char *cmdpath;
  char cmdline[1000]; // FIXME: buffer overrun
  char *tmp;

  if((s = LookupBuiltin(ast->cmd))) { 
    _alias_ok = 1; 
    return (*(s->ptr))(ast->args);
  }
  else if(_alias_ok && (p = ResolveAlias(ast->cmd))) {
    q = ll_to_charp(ast->args);
    tmp = emalloc(mystrlen(p) + mystrlen(q) + 2);
    strcpy(tmp, p);
    if(q != NULL) {
      strcat(tmp, " ");
      strcat(tmp, q);
    }
    setExecNext(tmp);
    free(p);
    free(q);
    _alias_ok = 0;
  }
  else {
    _alias_ok = 1;
    cmdpath = mystrdup(ast->cmd);
    p = strchr(cmdpath, '/');
    if(p == NULL) { free(cmdpath);
                    return _execute(ast,0); }
    else {
      q = cmdpath;
      while(p != NULL) { q = p+1; p = strchr(p+1, '/'); }
      cmdname = mystrdup(q);
      *q = 0;
      _cmdline = _getNewDir(cmdpath);
      sprintf(cmdline, "%s/%s", dir_ll_to_str(_cmdline), cmdname);
      free(cmdpath);

      return _execute(ast,1);
    }
  }
}

double _execute(ASTCommand *ast,int outp)
{
  glob_t globbuf;
  LinkedList *p;
  int glappend = 0;
  int status,ret,sig,job;
  REDIR *tmp,*redir;
  
  setCurrentPID(fork());
  if(getCurrentPID() == -1) {
    outputerr("Error!  Unable to create fork!\n");
    return RET_ERROR;
  }
  else if(getCurrentPID() == 0) { // Child process
    p = ast->args;

    tmp = ast->redir;
    while(tmp) {
      if(tmp->fdclose0 != 0) { 
              doutput("CLOSE %d in exec.c \"%s\" (child)\n", tmp->fdclose0,
                                                             ast->cmd);
              close(tmp->fdclose0);
      }
      if(tmp->filename) {
        if(tmp->stream == 0) 
          doutput("%s > %s\n", tmp->filename, ast->cmd);
        if(tmp->stream == 1)
          doutput("%s > %s\n", ast->cmd, tmp->filename);
        if(tmp->stream == 2)
          doutput("%s 2> %s\n", ast->cmd, tmp->filename);
      } else {
        if(tmp->stream == 0) 
          doutput("%d > %s\n", tmp->fd, ast->cmd);
        if(tmp->stream == 1)
          doutput("%s > %d\n", ast->cmd, tmp->fd);
        if(tmp->stream == 2)
          doutput("%s 2> %d\n", ast->cmd, tmp->fd);
      }
      if(tmp->fd > 0) { 
        dup2(tmp->fd,tmp->stream);
        doutput("CLOSE %d in exec.c \"%s\" (child)\n", tmp->fd, ast->cmd);
        close(tmp->fd);
      }
      tmp=tmp->next;
    }
    setCurrentAST(ast);
    if(p != NULL) {
      globbuf.gl_offs = 1;
      while(p != NULL) {
        glob(p->s, GLOB_DOOFFS | GLOB_NOCHECK | glappend, NULL, &globbuf); 
        glappend = GLOB_APPEND;
        p = p->next; 
      }
      globbuf.gl_pathv[0] = mystrdup(ast->cmd); // FIXME: free this?
      execvp(mystrdup(ast->cmd),&globbuf.gl_pathv[0]); 
    }
    else { execlp(mystrdup(ast->cmd),mystrdup(ast->cmd),NULL); }
    // FIXME: Why is this line here???
    if( outp ) outputerr("Exec error, errno %d\n", errno);
    outputerr("Exec error, errno %d (", errno);
    if( errno == ELIBBAD )            outputerr("ELIBBAD");
    else if( errno == EISDIR )        outputerr("EISDIR");
    else if( errno == EINVAL )        outputerr("EINVAL");
    else if( errno == EMFILE )        outputerr("EMFILE");
    else if( errno == ENFILE )        outputerr("ENFILE");
    else if( errno == EIO )           outputerr("EIO");
    else if( errno == ETXTBSY )       outputerr("ETXTBSY");
    else if( errno == ELOOP )         outputerr("ELOOP");
    else if( errno == EACCES )        outputerr("EACCES");
    else if( errno == ENOTDIR )       outputerr("ENOTDIR");
    else if( errno == ENOMEM )        outputerr("ENOMEM");
    else if( errno == ENOENT )        outputerr("ENOENT: \"%s\" does not exist",
                                                ast->cmd);
    else if( errno == ENAMETOOLONG )  outputerr("ENAMETOOLONG");
    else if( errno == EFAULT )        outputerr("EFAULT");
    else if( errno == ENOEXEC )       outputerr("ENOEXEC");
    else if( errno == E2BIG )         outputerr("E2BIG");
    else if( errno == EPERM )         outputerr("EPERM");
    outputerr(")\n");
    exit(1);
  } 
  else {
    // We are in the parent (shell) process
    // We wait for the process to execute if it is in the foreground
    // Otherwise we add it to the jobs list

        redir = ast->redir;
        while(redir) {
          if(redir->fdclose0 > 2) {
            doutput("CLOSE %d in exec.c (parent)\n", redir->fdclose0);
            close(redir->fdclose0);
          }
          if(redir->fdclose1 > 2) {
            doutput("CLOSE %d in exec.c (parent)\n", redir->fdclose1);
            close(redir->fdclose1);
          }
          redir=redir->next;
        }
    
    if(ast->bg) {
      job = AddJob(getCurrentPID(),getCurCMD(),ast);
      if(!ast->quiet) outputout("[%d] %d\n", job, getCurrentPID());
      setCurrentPID(0);
      return RET_SUCCESS;
    }
    else return _exec_wait(getCurrentPID());
  }
}

int _exec_wait(int mypid)
{
  int status,ret,sig;

  setCurrentPID(mypid);
  if ( (ret = waitpid(getCurrentPID(), &status, WUNTRACED)) < 0) {
    setCurrentPID(0);
    outputerr("waitpid error");
    return RET_ERROR;
  }
  else {
    setCurrentPID(0);
    if(WIFSIGNALED(status)) {
      sig = WTERMSIG(status);
      outputerr("Process %d terminated: %s\n", ret, getSigDesc(sig));
    }
    return WEXITSTATUS(status);
  }
}
