/* process.h -- Functions for dealing with processes. */

/* Copyright (C) 2003 Mike Thielman 
 *
 * This file is part of pr0nsh, the Porn Shell.
 *
 * Pr0nsh is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2, or (at your option) any later
 * version.
 *
 * Pr0nsh is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Pr0nsh; see the file COPYING.  If not, write to the Free Software
 * Foundation, 59 Temple Place, Suite 330, Boston, MA 02111 USA. */

#ifndef _PROCESS_H

#define _PROCESS_H

#include "util/llist.h"
#include "parse/ast.h"

typedef struct Job {
  int JobNum;
  int pid;
  char *cmdline;
  int status;
  int sig;           // if status is JOB_TERMINATED, by what signal?
  int changed;       // has status changed since last time printed
  ASTCommand *ast;   // The AST used to execute the job...we need to keep
                     // this around because we have to close it's file
                     // descriptors for redirection when it exits
  struct Job *next;
} Job;

// Possible status codes
#define JOB_RUNNING 0
#define JOB_DONE 1
#define JOB_STOPPED 2
#define JOB_SUSPENDED 3
#define JOB_TERMINATED 4

int AddJob(int pid, char *cmdline,ASTCommand *ast);
void SIGCHLDHandler(void);
int _lastJob;

Job *getNextJob(Job *job);
int isJobChanged(Job *job);
char *getJobStatusDesc(Job *job);
char *getJobCmdline(Job *job);
int getJobStatus(Job *job);
int getJobPID(Job *job);
int getJobNum(Job *job);
ASTCommand *getJobAST(Job *job);
void setJobChanged(Job *job);
void clearJobChanged(Job *job);
Job *getJobStructFromJobID(int jobid);
Job *getJobStructFromPID(int pid);
int getNumJobs();
Job *newJob(void);

#endif
