/* redir.c -- Functions for dealing with redirection. */

/* Copyright (C) 2003 Mike Thielman 
 *
 * This file is part of pr0nsh, the Porn Shell.
 *
 * Pr0nsh is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2, or (at your option) any later
 * version.
 *
 * Pr0nsh is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Pr0nsh; see the file COPYING.  If not, write to the Free Software
 * Foundation, 59 Temple Place, Suite 330, Boston, MA 02111 USA. */

#include <stdio.h>
#include <string.h>
#include "shell/redir.h"

static BUILTIN_REDIR _global_redir;

int getSTDINfd()
{
  return _global_redir.in;
}
int getSTDOUTfd()
{
  return _global_redir.out;
}
int getSTDERRfd()
{
  return _global_redir.err;
}
void setSTDINfd(int fd)
{
  _global_redir.in = fd;
}
void setSTDOUTfd(int fd)
{
  _global_redir.out = fd;
}
void setSTDERRfd(int fd)
{
  _global_redir.err = fd;
}

void freeREDIR(REDIR *redir)
{
  if(redir) {
    free(redir->filename);
    free(redir->next);
    free(redir);
  }
}

REDIR *copyREDIR(REDIR *redir)
{
  REDIR *copy = NULL;

  if(redir) {
    copy = newREDIR();
    
    copy->stream = redir->stream;
    copy->type = redir->type;
    copy->fd = redir->fd;
    copy->fdclose0 = redir->fdclose0;
    copy->fdclose1 = redir->fdclose1;

    if(redir->filename) copy->filename = strdup(redir->filename);
    if(redir->next)     copy->next = copyREDIR(redir->next);
  }

  return copy;
}

REDIR *newREDIR(void)
{
  REDIR *redir;

  redir = (REDIR *) emalloc(sizeof(REDIR));

  redir->filename = NULL;
  redir->next = NULL;

  return redir;
}
